import { from, Subject, map, BehaviorSubject, filter, switchMap, combineLatestWith } from 'rxjs';
import { takeUntilDestroyed } from '@angular/core/rxjs-interop';
import * as i0 from '@angular/core';
import { Injectable, Directive, Input, HostListener, inject } from '@angular/core';

class FeatureFlagStore {
    featureFlags$ = from([]);
}

class FeatureFlagsService {
    monitor = new Subject();
    monitor$;
    featureFlags = new Set();
    featureFlags$;
    constructor(featureStore) {
        this.monitor$ = this.monitor.asObservable();
        this.featureFlags$ = featureStore.featureFlags$.pipe(map(x => new Set(x)), takeUntilDestroyed());
        this.featureFlags$.subscribe(val => {
            this.featureFlags = val;
        });
    }
    /**
     * Returns true if the featureflag is turned off, false if it is turned off.
     * Preference is to use featureOff$ and use the observable when possibe.
     * @param {any} featureFlags:string|string[]
     * @param {any} operator:LogicalOperator='AND' operator to use, 'AND' or 'OR'
     * @returns {boolean} boolean true if the feature is off, false if the featureflag is not available
     */
    featureOff(featureFlags, operator = 'AND') {
        return !this.featureOn(featureFlags, operator);
    }
    /**
     * Check if the feature flag or set of featureflags of turned off
     * @param {any} featureFlags:string|string[]
     * @param {any} operator:LogicalOperator='AND' operator to use, 'AND' or 'OR'
     * @returns {boolean} Observable<boolean> true if the feature/feature set with condition is off
     */
    featureOff$(featureFlags, operator = 'AND') {
        return this.featureOn$(featureFlags, operator).pipe(map(val => !val));
    }
    /**
     * Returns true if the featureflag is turned on, false if it is turned on.
     * Preference is to use featureOn$ and use the observable when possibe.
     * @param {any} featureFlags:string|string[]
     * @param {any} operator:LogicalOperator='AND' operator to use, 'AND' or 'OR'
     * @returns {boolean} boolean true if the feature is on, false if the featureflag is not available
     */
    featureOn(featureFlags, operator = 'AND') {
        // if you don't specify anything, it's always on
        if (featureFlags === undefined || featureFlags === null) {
            return true;
        }
        const newArray = Array.isArray(featureFlags) ? featureFlags : [featureFlags];
        let foundFlags = [];
        for (const flag of newArray) {
            if (this.featureFlags.has(flag)) {
                foundFlags.push(flag);
            }
        }
        if (operator === 'OR' && foundFlags.length > 0) {
            for (let flag of foundFlags) {
                this.featureUsageTrace(flag);
            }
            return true;
        }
        else if (operator === 'AND' && foundFlags.length === newArray.length) {
            for (let flag of foundFlags) {
                this.featureUsageTrace(flag);
            }
            return true;
        }
        return false;
    }
    /**
     * Check if the feature flag or set of featureflags of turned on
     * @param {any} featureFlags:string|string[]
     * @param {any} operator:LogicalOperator='AND' operator to use, 'AND' or 'OR'
     * @returns {boolean} Observable<boolean> true if the feature/feature set with condition is on
     */
    featureOn$(featureFlags, operator = 'AND') {
        return this.featureFlags$.pipe(map(ffs => {
            // if you don't specify anything, it's always on
            if (featureFlags === undefined || featureFlags === null) {
                return true;
            }
            let matchingFlags = [];
            let foundFlags = [];
            if (typeof featureFlags == 'string') {
                foundFlags.push(featureFlags);
                if (ffs.has(featureFlags)) {
                    matchingFlags.push(featureFlags);
                }
            }
            else {
                const propNames = Object.getOwnPropertyNames(featureFlags);
                for (const propName of propNames) {
                    const s = `${String(propName)}:${featureFlags[propName]}`;
                    foundFlags.push(s);
                    if (ffs.has(s)) {
                        matchingFlags.push(s);
                    }
                }
            }
            if (operator === 'OR' && foundFlags.length > 0) {
                for (let flag of matchingFlags) {
                    this.featureUsageTrace(flag);
                }
                return true;
            }
            else if (operator === 'AND' && foundFlags.length === matchingFlags.length) {
                for (let flag of matchingFlags) {
                    this.featureUsageTrace(flag);
                }
                return true;
            }
            return false;
        }));
    }
    prev = [];
    /**
     * Trace feature usage
     * @param {any} featureTrace:string
     * @returns {void}
     */
    featureTrace(featureTrace) {
        if (featureTrace) {
            this.monitor.next(featureTrace);
        }
    }
    featureUsageTrace(featureTrace) {
        if (featureTrace) {
            if (!this.prev.includes(featureTrace)) {
                this.prev.push(featureTrace);
                this.monitor.next(featureTrace);
            }
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureFlagsService, deps: [{ token: FeatureFlagStore }], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureFlagsService });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureFlagsService, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: FeatureFlagStore }] });

class FeatureFlagTraceDirective {
    featureFlagService;
    features;
    set featureFlagTrace(features) {
        this.features = features;
    }
    constructor(featureFlagService) {
        this.featureFlagService = featureFlagService;
    }
    onClick() {
        if (!this.features) {
            return;
        }
        if (Array.isArray(this.features)) {
            for (const featureTrace of this.features) {
                this.featureFlagService.featureTrace(featureTrace);
            }
        }
        else {
            this.featureFlagService.featureTrace(this.features);
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureFlagTraceDirective, deps: [{ token: FeatureFlagsService }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.12", type: FeatureFlagTraceDirective, isStandalone: true, selector: "[featureFlagTrace]", inputs: { featureFlagTrace: "featureFlagTrace" }, host: { listeners: { "click": "onClick()" } }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureFlagTraceDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[featureFlagTrace]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: FeatureFlagsService }], propDecorators: { featureFlagTrace: [{
                type: Input
            }], onClick: [{
                type: HostListener,
                args: ['click']
            }] } });

const canActivateFeatureFlagGuard = (route, state) => {
    const featureFlags = route.data['featureFlags'];
    const logicalOperator = route.data['featureFlagsOperator'] ?? 'AND';
    return inject(FeatureFlagsService).featureOn$(featureFlags, logicalOperator);
};

class ShowIfFeatureFlagDirective {
    featureName = new BehaviorSubject(undefined);
    hasView = false;
    elseTemplateRef = null;
    set ifFeatureFlag(featureFlag) {
        this.featureName.next(featureFlag);
    }
    set ifFeatureFlagElse(templateRef) {
        this.elseTemplateRef = templateRef;
    }
    constructor(featureFlagService, templateRef, viewContainer) {
        this.featureName.pipe(filter(val => val !== undefined), switchMap((val) => featureFlagService.featureOn$(val)), takeUntilDestroyed())
            .subscribe(hasSomeFeatureOn => {
            if (this.hasView) {
                viewContainer.clear();
                this.hasView = false;
            }
            if (hasSomeFeatureOn) {
                viewContainer.createEmbeddedView(templateRef);
                this.hasView = true;
            }
            else if (this.elseTemplateRef) {
                viewContainer.createEmbeddedView(this.elseTemplateRef);
                this.hasView = true;
            }
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: ShowIfFeatureFlagDirective, deps: [{ token: FeatureFlagsService }, { token: i0.TemplateRef }, { token: i0.ViewContainerRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.12", type: ShowIfFeatureFlagDirective, isStandalone: true, selector: "[ifFeatureFlag]", inputs: { ifFeatureFlag: "ifFeatureFlag", ifFeatureFlagElse: "ifFeatureFlagElse" }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: ShowIfFeatureFlagDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ifFeatureFlag]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: FeatureFlagsService }, { type: i0.TemplateRef }, { type: i0.ViewContainerRef }], propDecorators: { ifFeatureFlag: [{
                type: Input
            }], ifFeatureFlagElse: [{
                type: Input
            }] } });

class ShowIfNotFeatureFlagDirective {
    featureName = new BehaviorSubject(undefined);
    set ifNotFeatureFlag(featureFlag) {
        this.featureName.next(featureFlag);
    }
    hasView = false;
    constructor(featureFlagService, templateRef, viewContainer) {
        this.featureName.pipe(filter(val => val !== undefined), switchMap((val) => featureFlagService.featureOff$(val)), takeUntilDestroyed())
            .subscribe(hasSomeFeatureOff => {
            if (hasSomeFeatureOff && !this.hasView) {
                viewContainer.createEmbeddedView(templateRef);
                this.hasView = true;
            }
            else if (!hasSomeFeatureOff && this.hasView) {
                viewContainer.clear();
                this.hasView = false;
            }
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: ShowIfNotFeatureFlagDirective, deps: [{ token: FeatureFlagsService }, { token: i0.TemplateRef }, { token: i0.ViewContainerRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.12", type: ShowIfNotFeatureFlagDirective, isStandalone: true, selector: "[ifNotFeatureFlag]", inputs: { ifNotFeatureFlag: "ifNotFeatureFlag" }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: ShowIfNotFeatureFlagDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ifNotFeatureFlag]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: FeatureFlagsService }, { type: i0.TemplateRef }, { type: i0.ViewContainerRef }], propDecorators: { ifNotFeatureFlag: [{
                type: Input
            }] } });

class FeatureDirective {
    featureFlagService;
    features;
    set feature(features) {
        this.features = features;
    }
    constructor(featureFlagService) {
        this.featureFlagService = featureFlagService;
    }
    ngAfterViewInit() {
        if (!this.features) {
            return;
        }
        if (Array.isArray(this.features)) {
            for (const featureTrace of this.features) {
                this.featureFlagService.featureTrace(featureTrace);
            }
        }
        else {
            this.featureFlagService.featureTrace(this.features);
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureDirective, deps: [{ token: FeatureFlagsService }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.12", type: FeatureDirective, isStandalone: true, selector: "[feature]", inputs: { feature: "feature" }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[feature]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: FeatureFlagsService }], propDecorators: { feature: [{
                type: Input
            }] } });

class FeatureFlagClassDirective {
    _ngEl;
    _renderer;
    classObservable = new BehaviorSubject([]);
    constructor(_ngEl, _renderer, featureFlagService) {
        this._ngEl = _ngEl;
        this._renderer = _renderer;
        featureFlagService.featureFlags$
            .pipe(combineLatestWith(this.classObservable), map(([flags, classO]) => {
            const toEnable = classO.filter(i => flags.has(i.toggle));
            const toDisable = classO.filter(i => !flags.has(i.toggle));
            return { enable: toEnable, disable: toDisable };
        }), takeUntilDestroyed())
            .subscribe(flags => {
            flags.enable.forEach(i => this.toggleClass(i.class, true));
            flags.disable.forEach(i => this.toggleClass(i.class, false));
        });
    }
    set klass(value) {
    }
    set ffClass(value) {
        const dd = [];
        if (value !== null && value !== undefined) {
            Object.keys(value).forEach(klass => dd.push({ class: klass, toggle: value[klass] }));
        }
        this.classObservable.next(dd);
    }
    toggleClass(klass, enabled) {
        const classs = new String(klass).trim();
        if (classs) {
            classs.split(/\s+/g).forEach(klass => {
                if (enabled) {
                    this._renderer.addClass(this._ngEl.nativeElement, klass);
                }
                else {
                    this._renderer.removeClass(this._ngEl.nativeElement, klass);
                }
            });
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureFlagClassDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: FeatureFlagsService }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.12", type: FeatureFlagClassDirective, isStandalone: true, selector: "[ffClass]", inputs: { klass: ["class", "klass"], ffClass: "ffClass" }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureFlagClassDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ffClass]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: FeatureFlagsService }], propDecorators: { klass: [{
                type: Input,
                args: ['class']
            }], ffClass: [{
                type: Input,
                args: ['ffClass']
            }] } });

class FeatureToggleStore {
    featureToggles = new BehaviorSubject([]);
    featureToggles$ = this.featureToggles.asObservable();
    toggle(feature, state = undefined) {
        const toggles = this.featureToggles.value;
        const toggle = this.findToggle(toggles, feature);
        if (toggle) {
            toggle.enabled = state ?? !toggle.enabled;
            this.featureToggles.next(toggles);
        }
    }
    injectToggle(featureToggle) {
        const toggles = this.featureToggles.value;
        toggles.push(featureToggle);
        this.featureToggles.next(toggles);
    }
    withdrawToggle(feature) {
        const toggles = this.featureToggles.value;
        const toggle = this.findToggle(toggles, feature);
        if (toggle) {
            this.featureToggles.next(toggles.filter(t => t !== toggle));
        }
    }
    findToggle(toggles, feature) {
        if (typeof feature == 'string') {
            return toggles.find(t => t.name == feature);
        }
        else {
            return toggles.find(t => t.name == feature.name);
        }
    }
}

class FeatureToggleService {
    featureToggleStore;
    featureToggles$;
    constructor(featureToggleStore) {
        this.featureToggleStore = featureToggleStore;
        this.featureToggles$ = featureToggleStore.featureToggles$;
    }
    toggleFeature(feature) {
        this.featureToggleStore.toggle(feature);
    }
    enableToggle(feature) {
        this.featureToggleStore.toggle(feature, true);
    }
    disableToggle(feature) {
        this.featureToggleStore.toggle(feature, false);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureToggleService, deps: [{ token: FeatureToggleStore }], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureToggleService, providedIn: 'root' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.12", ngImport: i0, type: FeatureToggleService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: FeatureToggleStore }] });

/*
 * Public API Surface of angular-feature-flags
 */

/**
 * Generated bundle index. Do not edit.
 */

export { FeatureDirective, FeatureFlagClassDirective, FeatureFlagStore, FeatureFlagTraceDirective, FeatureFlagsService, FeatureToggleService, FeatureToggleStore, ShowIfFeatureFlagDirective, ShowIfNotFeatureFlagDirective, canActivateFeatureFlagGuard };
//# sourceMappingURL=eriklieben-angular-feature-flags.mjs.map
